@@ -63,13 +63,15 @@ class Register(ABC, Generic[T]):
     """
 
     def __init__(self, registry: _Registry, instantiation: Instantiation, param_names_as_bindings: bool):
+        self._is_valid: bool = True
         self._key: RegistryKey = None
         self._registry: _Registry = registry
         self._instantiation: Instantiation = instantiation
         self._param_names_as_bindings: bool = param_names_as_bindings
         self._given_positional_params: Sequence = ()
         self._given_keyword_params: Dict = {}
-        self._given_name_bindings: Dict = {}
+        self._given_positional_name_bindings: Sequence = ()
+        self._given_keyword_name_bindings: Dict = {}
         self._cached_instance: T = None
 
     @abstractmethod
@@ -80,6 +82,16 @@ class Register(ABC, Generic[T]):
     def _factory_method(self) -> Callable[..., T]:
         pass
 
+    def _invalidate(self):
+        self._is_valid = False
+        self._registry.pop(self._key, None)
+
+    def _check_validity(self):
+        if not self._is_valid:
+            error_msg = f'This Register is not valid anymore. No operations are allowed.'
+            _logger.error(error_msg)
+            raise DiContainerError(error_msg)
+
     def _raise_bad_operation_error(self):
         operation = inspect.stack()[1].function
         raise DiContainerError(f'The operation "{operation}" is not supported on "{self.__class__.__name__}" objects.')
@@ -94,6 +106,14 @@ class Register(ABC, Generic[T]):
             _logger.error(error_msg)
             raise TypeError(error_msg)
 
+    def _check_only_one_type_of_positional_input(self):
+        if self._given_positional_params and self._given_positional_name_bindings:
+            self._invalidate()
+            error_msg = f'Cannot define both positional params ({self._given_positional_params}) ' \
+                        f'and positional name bindings ({self._given_positional_name_bindings}).'
+            _logger.error(error_msg)
+            raise DiContainerError(error_msg)
+
     def with_params(self, *positional_params, **keyword_params) -> 'Register[T]':
         """
         Assigns values to arguments of the registered object, provided it's :func:`callable` (types, functions etc.).
@@ -102,31 +122,41 @@ class Register(ABC, Generic[T]):
         :param keyword_params: keyword=value assignments.
         :return: The Register object itself, for fluent interfacing.
         """
+        self._check_validity()
         self._given_positional_params = positional_params
+        self._check_only_one_type_of_positional_input()
         self._given_keyword_params = keyword_params
         return self
 
-    def with_name_bindings(self, **name_bindings) -> 'Register[T]':
+    def with_name_bindings(self, *positional_name_bindings, **keyword_name_bindings) -> 'Register[T]':
         """
         Assigns name bindings to arguments of the registered object,
         provided it's :func:`callable` (types, functions etc.).
 
-        :param name_bindings: keyword=name_binding assignments,
+        :param positional_name_bindings: positional name_binding assignments,
+        where :code:`name_binding` is a name to which a dependency is registered.
+        :param keyword_name_bindings: keyword=name_binding assignments,
         where :code:`name_binding` is a name to which a dependency is registered.
         :return: The Register object itself, for fluent interfacing.
         """
-        self._given_name_bindings = name_bindings
+        self._check_validity()
+        self._given_positional_name_bindings = positional_name_bindings
+        self._check_only_one_type_of_positional_input()
+        self._given_keyword_name_bindings = keyword_name_bindings
         return self
 
+    @staticmethod
+    def _str_key(key: Any) -> str:
+        return key.__name__ if hasattr(key, '__name__') else key
+
     def _register_to_key(self, register_key: RegistryKey, type_to_check_against: type, replace: bool = False):
         if self._key is not None:
-            error_msg = f'This Register record is already bound to ' \
-                        f'"{self._key.__name__ if callable(self._key) else self._key}".' \
+            error_msg = f'This Register record is already bound to "{self._str_key(self._key)}".' \
                         ' Probably invoked to_type/to_name more than once.'
             _logger.error(error_msg)
             raise DiContainerError(error_msg)
         if register_key in self._registry and not replace:
-            error_msg = f'The register key "{register_key.__name__ if callable(register_key) else register_key}"' \
+            error_msg = f'The register key "{self._str_key(register_key)}"' \
                         f' is already bound to "{self._registry[register_key]._type_to_check().__name__}"' \
                         f' and replace was not requested (replace == False).'
             _logger.error(error_msg)
@@ -148,6 +178,7 @@ class Register(ABC, Generic[T]):
         :raises ValueError: If the given dependency type has a registration already and replace is False.
         :raises TypeError: If the object in this Register is not a subclass of the given dependency type.
         """
+        self._check_validity()
         self._register_to_key(dependency_type, dependency_type, replace)
         return self
 
@@ -164,6 +195,7 @@ class Register(ABC, Generic[T]):
         :raises ValueError: If the given name has a registration already and replace is False.
         :raises TypeError: If the object in this Register is not a subclass of the given base type, if given.
         """
+        self._check_validity()
         self._register_to_key(name, base_type, replace)
         return self
 
@@ -230,8 +262,15 @@ class Register(ABC, Generic[T]):
                 has_given_value = False
                 given_value = None
 
-            has_name_binding = arg in self._given_name_bindings
-            name_binding = self._given_name_bindings.get(arg, None)
+            if i <= len(self._given_positional_name_bindings) - 1:  # is there a given value in the i'th place?
+                has_name_binding = True
+                name_binding = self._given_positional_name_bindings[i]
+            elif arg in self._given_keyword_name_bindings:
+                has_name_binding = True
+                name_binding = self._given_keyword_name_bindings.get(arg, None)
+            else:
+                has_name_binding = False
+                name_binding = None
 
             default_index = i - (len(params.args) - len(params.defaults or ()))  # default matching from the end
             has_default = default_index >= 0
@@ -255,6 +294,15 @@ class Register(ABC, Generic[T]):
             else:
                 error_desc = 'Too many positional arguments given, or vararg (*args) is missing'
                 errors.append(error_desc)
+        elif len(self._given_positional_name_bindings) > len(params.args):
+            if params.varargs is not None:
+                arg_values.extend(self._given_positional_params[len(params.args):])
+                values = [self._registry[name_binding]._resolve_recursive(errors, registers_in_resolving_process, prefer_defaults=False)
+                          for name_binding in self._given_positional_name_bindings[len(params.args):]]
+                arg_values.extend(values)
+            else:
+                error_desc = 'Too many positional name bindings given, or vararg (*args) is missing'
+                errors.append(error_desc)
 
         # handle keyword arguments
         for kwarg in params.kwonlyargs:
@@ -265,8 +313,8 @@ class Register(ABC, Generic[T]):
                 has_given_value = False
                 given_value = None
 
-            has_name_binding = kwarg in self._given_name_bindings
-            name_binding = self._given_name_bindings.get(kwarg, None)
+            has_name_binding = kwarg in self._given_keyword_name_bindings
+            name_binding = self._given_keyword_name_bindings.get(kwarg, None)
 
             has_default = kwarg in (params.kwonlydefaults or ())
             default = params.kwonlydefaults[kwarg] if has_default else None
@@ -282,7 +330,7 @@ class Register(ABC, Generic[T]):
             kwarg_values[kwarg] = kwarg_value
             errors.extend(param_errors)
 
-        # handle varkw (**kwargs)
+        # handle varkw (**kwargs) with unused given keyword params
         unused_keywords = set(self._given_keyword_params.keys()) - set(params.args) - set(params.kwonlyargs)
         if unused_keywords:
             if params.varkw is not None:
@@ -291,6 +339,23 @@ class Register(ABC, Generic[T]):
                 error_desc = f'The given keyword arguments {unused_keywords} are either wrong, ' \
                              'or varkw (**kwargs) is missing'
                 errors.append(error_desc)
+
+        # handle varkw (**kwargs) with unused given name bindings
+        unused_name_bindings = set(self._given_keyword_name_bindings.keys()) - set(params.args) - set(params.kwonlyargs)
+        if unused_name_bindings:
+            if params.varkw is not None:
+                for kwarg, name_binding in self._given_keyword_name_bindings.items():
+                    kwarg_value, param_errors = self._resolve_param(kwarg, registers_in_resolving_process,
+                                                                    has_given_value=False, given_value=None,
+                                                                    has_name_binding=True, name_binding=name_binding,
+                                                                    has_default=False, default=None, prefer_defaults=prefer_defaults,
+                                                                    has_annotation=False, annotation=None)
+                    kwarg_values[kwarg] = kwarg_value
+                    errors.extend(param_errors)
+            else:
+                error_desc = f'The given name bindings {unused_name_bindings} are either wrong, ' \
+                             'or varkw (**kwargs) is missing'
+                errors.append(error_desc)
         return arg_values, kwarg_values, errors
 
     def _resolve(self, registers_in_resolving_process: Deque['Register'], prefer_defaults: bool) -> Tuple[Optional[T], List[str]]:
@@ -299,8 +364,14 @@ class Register(ABC, Generic[T]):
         """
         a_callable = self._factory_method()
         if self in registers_in_resolving_process:
-            errors = [f"{a_callable.__name__}'s registration to {self._key if type(self._key) is str else self._key.__name__} "
-                      'has a circular dependency on itself. Set the problematic parameter explicitly.']
+            occurrence_index = registers_in_resolving_process.index(self)
+            circle = f'{self._str_key(registers_in_resolving_process[occurrence_index]._factory_method())} ({self._str_key(self._key)}) --> '
+            occurrence_index += 1
+            circle += ' --> '.join(self._str_key(registers_in_resolving_process[i]._factory_method())
+                                   for i in range(occurrence_index, len(registers_in_resolving_process)))
+            circle += f' --> {self._str_key(self._key)}'
+            errors = [f"{a_callable.__name__}'s registration to {self._str_key(self._key)} "
+                      f'has a circular dependency on itself: {circle}. Set the problematic parameter explicitly.']
             value = None
         else:
             registers_in_resolving_process.append(self)
@@ -338,6 +409,7 @@ class Register(ABC, Generic[T]):
         :return: The resolved value.
         :raises DiContainerError: If errors occurred during the resolving process.
         """
+        self._check_validity()
         return self._resolve_recursive(prefer_defaults=prefer_defaults, errors=[], registers_in_resolving_process=deque())
 
 
@@ -375,7 +447,7 @@ class ValueRegister(Register):
     def with_params(self, *positional_params, **keyword_params):
         self._raise_bad_operation_error()
 
-    def with_name_bindings(self, **name_bindings):
+    def with_name_bindings(self, **keyword_name_bindings):
         self._raise_bad_operation_error()
 
 
